from __future__ import annotations
import polars as pl
from polars._typing import IntoExpr, PolarsDataType


def parse_into_expr(
    expr: IntoExpr,
    *,
    str_as_lit: bool = False,
    list_as_lit: bool = True,
    dtype: PolarsDataType | None = None,
) -> pl.Expr:
    """
    Parse a single input into an expression.

    Parameters
    ----------
    expr
        The input to be parsed as an expression.
    str_as_lit
        Interpret string input as a string literal. If set to `False` (default),
        strings are parsed as column names.
    list_as_lit
        Interpret list input as a lit literal, If set to `False`,
        lists are parsed as `Series` literals.
    dtype
        If the input is expected to resolve to a literal with a known dtype, pass
        this to the `lit` constructor.

    Returns
    -------
    polars.Expr
    """
    if isinstance(expr, pl.Expr):
        pass
    elif isinstance(expr, str) and not str_as_lit:
        expr = pl.col(expr)
    elif isinstance(expr, list) and not list_as_lit:
        expr = pl.lit(pl.Series(expr), dtype=dtype)
    else:
        expr = pl.lit(expr, dtype=dtype)

    return expr



import polars as pl
from polars_strsim import levenshtein, jaro, jaro_winkler, jaccard, sorensen_dice

df = pl.DataFrame(
    {
        "name_a": ["phillips", "phillips", "", "", None, None],
        "name_b": ["phillips", "philips", "phillips", "", "phillips", None],
    }
).with_columns(
    levenshtein=levenshtein("name_a", "name_b"),
    jaro=jaro("name_a", "name_b"),
    jaro_winkler=jaro_winkler("name_a", "name_b"),
    jaccard=jaccard("name_a", "name_b"),
    sorensen_dice=sorensen_dice("name_a", "name_b"),
)

print(df)



from pathlib import Path
import polars as pl
from polars.plugins import register_plugin_function
from polars._typing import IntoExpr
from polars_strsim.utils import parse_into_expr


def levenshtein(expr: IntoExpr, other: IntoExpr) -> pl.Expr:
    expr = parse_into_expr(expr, dtype=pl.Utf8)
    other = parse_into_expr(other, dtype=pl.Utf8)
    return register_plugin_function(
        plugin_path=Path(__file__).parent,
        function_name="levenshtein",
        args=[expr, other],
        is_elementwise=True,
    )


def jaro(expr: IntoExpr, other: IntoExpr) -> pl.Expr:
    expr = parse_into_expr(expr, dtype=pl.Utf8)
    other = parse_into_expr(other, dtype=pl.Utf8)
    return register_plugin_function(
        plugin_path=Path(__file__).parent,
        function_name="jaro",
        args=[expr, other],
        is_elementwise=True,
    )


def jaro_winkler(expr: IntoExpr, other: IntoExpr) -> pl.Expr:
    expr = parse_into_expr(expr, dtype=pl.Utf8)
    other = parse_into_expr(other, dtype=pl.Utf8)
    return register_plugin_function(
        plugin_path=Path(__file__).parent,
        function_name="jaro_winkler",
        args=[expr, other],
        is_elementwise=True,
    )


def jaccard(expr: IntoExpr, other: IntoExpr) -> pl.Expr:
    expr = parse_into_expr(expr, dtype=pl.Utf8)
    other = parse_into_expr(other, dtype=pl.Utf8)
    return register_plugin_function(
        plugin_path=Path(__file__).parent,
        function_name="jaccard",
        args=[expr, other],
        is_elementwise=True,
    )


def sorensen_dice(expr: IntoExpr, other: IntoExpr) -> pl.Expr:
    expr = parse_into_expr(expr, dtype=pl.Utf8)
    other = parse_into_expr(other, dtype=pl.Utf8)
    return register_plugin_function(
        plugin_path=Path(__file__).parent,
        function_name="sorensen_dice",
        args=[expr, other],
        is_elementwise=True,
    )


__all__ = [
    "levenshtein",
    "jaro",
    "jaro_winkler",
    "jaccard",
    "sorensen_dice",
]
